@@ -34,6 +34,11 @@
 #include <gconf/gconf-client.h>
 #include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-ui-init.h>
+#include <libgnomevfs/gnome-vfs-async-ops.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+
+#include "file-transfer-dialog.h"
 
 #include "gs-job.h"
 #include "gs-prefs.h" /* for GS_MODE enum */
@@ -66,6 +71,18 @@ static GConfEnumStringPair mode_enum_map [] = {
        { 0, NULL }
 };
 
+/* Drag and drop info */
+enum {
+        TARGET_URI_LIST,
+        TARGET_NS_URL
+};
+
+static GtkTargetEntry drop_types [] =
+{
+        { "text/uri-list", 0, TARGET_URI_LIST },
+        { "_NETSCAPE_URL", 0, TARGET_NS_URL }
+};
+
 static GladeXML *xml = NULL;
 static GSJob    *job = NULL;
 
@@ -273,6 +290,12 @@ populate_model (GtkTreeStore *store)
                             NAME_COLUMN, "__blank-only",
                             -1);
 
+        gtk_tree_store_append (store, &iter, NULL);
+        gtk_tree_store_set (store, &iter,
+                            LABEL_COLUMN, NULL,
+                            NAME_COLUMN, "__separator",
+                            -1);
+
         themes = get_theme_list ();
 
         if (! themes)
@@ -349,6 +372,11 @@ compare_theme  (GtkTreeModel *model,
         gtk_tree_model_get (model, a, LABEL_COLUMN, &label_a, -1);
         gtk_tree_model_get (model, b, LABEL_COLUMN, &label_b, -1);
 
+        if (! name_a)
+                return 1;
+        else if (! name_b)
+                return -1;
+
         if (strcmp (name_a, "__disabled") == 0)
                 return -1;
         else if (strcmp (name_b, "__disabled") == 0)
@@ -357,6 +385,15 @@ compare_theme  (GtkTreeModel *model,
                 return -1;
         else if (strcmp (name_b, "__blank-only") == 0)
                 return 1;
+        else if (strcmp (name_a, "__separator") == 0)
+                return -1;
+        else if (strcmp (name_b, "__separator") == 0)
+                return 1;
+
+        if (! label_a)
+                return 1;
+        else if (! label_b)
+                return -1;
 
         result = strcmp (label_a, label_b);
 
@@ -368,6 +405,23 @@ compare_theme  (GtkTreeModel *model,
         return result;
 }
 
+static gboolean
+separator_func (GtkTreeModel *model,
+                GtkTreeIter  *iter,
+                gpointer      data)
+{
+        int   column = GPOINTER_TO_INT (data);
+        char *text;
+        
+        gtk_tree_model_get (model, iter, column, &text, -1);
+        
+        if (text && strcmp (text, "__separator") == 0)
+                return TRUE;
+        
+        g_free (text);
+
+        return FALSE;
+}
 
 static void
 setup_treeview (GtkWidget *tree,
@@ -403,6 +457,11 @@ setup_treeview (GtkWidget *tree,
                                               0,
                                               GTK_SORT_ASCENDING);
 
+        gtk_tree_view_set_row_separator_func (GTK_TREE_VIEW (tree),
+                                              separator_func,
+                                              GINT_TO_POINTER (NAME_COLUMN),
+                                              NULL);
+
         select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
         gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
         g_signal_connect (G_OBJECT (select), "changed",
@@ -453,6 +512,247 @@ setup_treeview_selection (GtkWidget *tree)
         g_free (theme);
 }
 
+static void
+reload_themes (void)
+{
+        GtkWidget    *treeview;
+        GtkTreeModel *model;
+
+        treeview = glade_xml_get_widget (xml, "savers_treeview");
+        model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
+        gtk_tree_store_clear (GTK_TREE_STORE (model));
+        populate_model (GTK_TREE_STORE (model));
+
+        gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
+                                 GTK_TREE_MODEL (model));
+}
+
+static void
+transfer_done_cb (GtkWidget *dialog,
+                  char      *path)
+{
+
+        g_free (path);
+
+        gtk_widget_destroy (dialog);
+
+        reload_themes ();
+}
+
+static void
+transfer_cancel_cb (GtkWidget *dialog,
+                    char      *path)
+{
+        gnome_vfs_unlink (path);
+        g_free (path);
+
+        gtk_widget_destroy (dialog);
+}
+
+static void
+theme_installer_run (GtkWidget *parent,
+                     char      *filename)
+{
+        GtkWidget    *dialog;
+        GnomeVFSURI  *src_uri;
+        GList        *src, *target;
+        char         *target_path;
+        char         *user_dir;
+        char         *base;
+
+        src_uri = gnome_vfs_uri_new (filename);
+        src = g_list_append (NULL, src_uri);
+
+        user_dir = g_build_filename (g_get_user_data_dir (), "gnome-screensaver", "themes", NULL);
+        base = gnome_vfs_uri_extract_short_name (src_uri);
+
+        target_path = NULL;
+
+        while (TRUE) {
+                char      *file_tmp;
+                GtkWidget *dialog;
+                int        len = strlen (base);
+
+                if (base && len > 4 && (!strcmp (base + len - 4, ".xml"))) {
+                        file_tmp = g_strdup_printf ("screensaver-theme-%d.xml", rand ());
+                } else {
+                        dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
+                                                         GTK_DIALOG_MODAL,
+                                                         GTK_MESSAGE_ERROR,
+                                                         GTK_BUTTONS_OK,
+                                                         _("Invalid screensaver theme"));
+                        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+                                                                  "%s",
+                                                                  _("This file does not appear to be a valid screensaver theme."));
+                        gtk_window_set_title (GTK_WINDOW (dialog), "");
+                        gtk_window_set_icon_name (GTK_WINDOW (dialog), "screensaver");
+
+                        gtk_dialog_run (GTK_DIALOG (dialog));
+                        gtk_widget_destroy (dialog);
+                        g_free (target_path);
+                        return;
+                }
+
+                target_path = g_build_filename (user_dir, file_tmp, NULL);
+
+                g_free (file_tmp);
+                if (! gnome_vfs_uri_exists (gnome_vfs_uri_new (target_path)))
+                        break;
+        }
+                
+        g_free (user_dir);
+
+        target = g_list_append (NULL, gnome_vfs_uri_new (target_path));
+
+        dialog = file_transfer_dialog_new ();
+        gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
+        gtk_window_set_icon_name (GTK_WINDOW (dialog), "screensaver");
+
+        file_transfer_dialog_wrap_async_xfer (FILE_TRANSFER_DIALOG (dialog),
+                                              src, target,
+                                              GNOME_VFS_XFER_RECURSIVE,
+                                              GNOME_VFS_XFER_ERROR_MODE_QUERY,
+                                              GNOME_VFS_XFER_OVERWRITE_MODE_QUERY,
+                                              GNOME_VFS_PRIORITY_DEFAULT);
+        gnome_vfs_uri_list_unref (src);
+        gnome_vfs_uri_list_unref (target);
+
+        g_signal_connect (dialog, "cancel",
+                          G_CALLBACK (transfer_cancel_cb), target_path);
+        g_signal_connect (dialog, "done",
+                          G_CALLBACK (transfer_done_cb), target_path);
+
+        gtk_widget_show (dialog);
+}
+
+/* Callback issued during drag movements */
+static gboolean
+drag_motion_cb (GtkWidget      *widget,
+                GdkDragContext *context,
+                int             x,
+                int             y,
+                guint           time,
+                gpointer        data)
+{
+        return FALSE;
+}
+
+/* Callback issued during drag leaves */
+static void
+drag_leave_cb (GtkWidget      *widget,
+               GdkDragContext *context,
+	       guint           time,
+               gpointer        data)
+{
+        gtk_widget_queue_draw (widget);
+}
+
+/* Callback issued on actual drops. Attempts to load the file dropped. */
+static void
+drag_data_received_cb (GtkWidget        *widget,
+                       GdkDragContext   *context,
+                       int               x,
+                       int               y,
+                       GtkSelectionData *selection_data,
+                       guint             info,
+                       guint             time,
+                       gpointer          data)
+{
+        GtkWidget *dialog;
+        GList     *uris;
+        char      *filename = NULL;
+
+        if (!(info == TARGET_URI_LIST || info == TARGET_NS_URL))
+                return;
+
+        uris = gnome_vfs_uri_list_parse ((char *) selection_data->data);
+        if (uris != NULL && uris->data != NULL) {
+                GnomeVFSURI *uri = (GnomeVFSURI *) uris->data;
+
+                if (gnome_vfs_uri_is_local (uri))
+                        filename = gnome_vfs_unescape_string (
+                                        gnome_vfs_uri_get_path (uri),
+                                        G_DIR_SEPARATOR_S);
+                else
+                        filename = gnome_vfs_unescape_string (
+                                        gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE),
+                                        G_DIR_SEPARATOR_S);
+
+                gnome_vfs_uri_list_unref (uris);
+        }
+
+        dialog = glade_xml_get_widget (xml, "prefs_dialog");
+        theme_installer_run (dialog, filename);
+
+        g_free (filename);
+}
+
+/* Adapted from totem_time_to_string_text */
+static char *
+time_to_string_text (long time)
+{
+        char *secs, *mins, *hours, *string;
+        int   sec, min, hour;
+
+        sec = time % 60;
+        time = time - sec;
+        min = (time % (60 * 60)) / 60;
+        time = time - (min * 60);
+        hour = time / (60 * 60);
+
+        hours = g_strdup_printf (ngettext ("%d hour",
+                                           "%d hours", hour), hour);
+
+        mins = g_strdup_printf (ngettext ("%d minute",
+                                          "%d minutes", min), min);
+
+        secs = g_strdup_printf (ngettext ("%d second",
+                                          "%d seconds", sec), sec);
+
+        if (hour > 0) {
+                if (sec > 0) {
+                        /* hour:minutes:seconds */
+                        string = g_strdup_printf (_("%s %s %s"), hours, mins, secs);
+                } else if (min > 0) {
+                        /* hour:minutes */
+                        string = g_strdup_printf (_("%s %s"), hours, mins);
+                } else {
+                        /* hour */
+                        string = g_strdup_printf (_("%s"), hours);
+                }
+        } else if (min > 0) {
+                if (sec > 0) {
+                        /* minutes:seconds */
+                        string = g_strdup_printf (_("%s %s"), mins, secs);
+                } else {
+                        /* minutes */
+                        string = g_strdup_printf (_("%s"), mins);
+                }
+        } else if (sec > 0) {
+                /* seconds */
+                string = g_strdup_printf (_("%s"), secs);
+        } else {
+                /* 0 seconds */
+                string = g_strdup (_("0 seconds"));
+        }
+
+        g_free (hours);
+        g_free (mins);
+        g_free (secs);
+
+        return string;
+}
+
+static char *
+format_value_callback_time (GtkScale *scale,
+                            gdouble   value)
+{
+        if (value == 0)
+                return g_strdup_printf (_("Never"));
+
+        return time_to_string_text (value * 60.0);
+}
+
 static void
 init_capplet (void)
 {
@@ -502,9 +802,23 @@ init_capplet (void)
         gtk_range_set_value (GTK_RANGE (blank_delay_hscale), blank_delay);
         if (! is_writable)
                 gtk_widget_set_sensitive (blank_delay_hbox, FALSE);
+        g_signal_connect (blank_delay_hscale, "format-value",
+                          G_CALLBACK (format_value_callback_time), NULL);
 
         gtk_window_set_icon_name (GTK_WINDOW (dialog), "screensaver");
 
+
+        gtk_drag_dest_set (dialog, GTK_DEST_DEFAULT_ALL,
+                           drop_types, G_N_ELEMENTS (drop_types),
+                           GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_MOVE);
+
+        g_signal_connect (dialog, "drag-motion",
+                          G_CALLBACK (drag_motion_cb), NULL);
+        g_signal_connect (dialog, "drag-leave",
+                          G_CALLBACK (drag_leave_cb), NULL);
+        g_signal_connect (dialog, "drag-data-received",
+                          G_CALLBACK (drag_data_received_cb), NULL);
+
         gtk_widget_show_all (dialog);
 
         preview_clear (preview);
@@ -513,7 +827,7 @@ init_capplet (void)
         /* Add user configuration path */
         /* FIXME: disable this if locked down */
         path = g_build_filename (g_get_user_data_dir (), "gnome-screensaver", "themes", NULL);
-        g_message ("Adding theme path: %s", path);
+
         gs_job_prepend_theme_path (job, path);
         g_free (path);
 
