@@ -37,6 +37,7 @@
 #include "gs-lock-plug.h"
 
 #include "passwd.h"
+
 #include "fusa-manager.h"
 
 enum { 
@@ -66,6 +67,7 @@ struct GSLockPlugPrivate
         GtkWidget   *switch_button;
 
         gboolean     caps_lock_on;
+        gboolean     switch_enabled;
         gboolean     logout_enabled;
 
         guint        timeout;
@@ -92,7 +94,8 @@ enum {
 
 enum {
         PROP_0,
-        PROP_LOGOUT_ENABLED
+        PROP_LOGOUT_ENABLED,
+        PROP_SWITCH_ENABLED
 };
 
 static GObjectClass *parent_class = NULL;
@@ -173,7 +176,7 @@ gs_lock_plug_response (GSLockPlug *plug,
 
                 if (current_page == 0) {
                         gtk_widget_set_sensitive (plug->priv->password_entry, FALSE);
-                        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar), _("Verifying password..."));
+                        gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar), _("Checking password..."));
 
                         plug->priv->password_check_idle_id = g_idle_add ((GSourceFunc)password_check_idle_cb,
                                                                          plug);
@@ -223,6 +226,7 @@ monitor_progress (GSLockPlug *plug)
         GTimeVal now;
         glong    elapsed;
         glong    remaining;
+        glong    secs;
         gdouble  fraction;
         char    *message;
 
@@ -236,8 +240,10 @@ monitor_progress (GSLockPlug *plug)
 
         gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (plug->priv->progress_bar), fraction);
 
-        message = g_strdup_printf (_("About %ld seconds left"),
-                                   remaining / 1000 + 1);
+        secs = remaining / 1000 + 1;
+        message = g_strdup_printf (ngettext ("About %ld second left",
+                                             "About %ld seconds left", secs),
+                                   secs);
         gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar),
                                    message);
         g_free (message);
@@ -329,6 +335,24 @@ gs_lock_plug_set_logout_enabled (GSLockPlug *plug,
                 gtk_widget_hide (plug->priv->logout_button);
 }
 
+static void
+gs_lock_plug_set_switch_enabled (GSLockPlug *plug,
+                                 gboolean    switch_enabled)
+{
+        g_return_if_fail (GS_LOCK_PLUG (plug));
+
+        if (plug->priv->switch_enabled == switch_enabled)
+                return;
+
+        plug->priv->switch_enabled = switch_enabled;
+        g_object_notify (G_OBJECT (plug), "switch-enabled");
+
+        if (switch_enabled)
+                gtk_widget_show (plug->priv->switch_button);
+        else
+                gtk_widget_hide (plug->priv->switch_button);
+}
+
 static void
 gs_lock_plug_set_property (GObject            *object,
                            guint               prop_id,
@@ -343,6 +367,9 @@ gs_lock_plug_set_property (GObject            *object,
         case PROP_LOGOUT_ENABLED:
                 gs_lock_plug_set_logout_enabled (self, g_value_get_boolean (value));
                 break;
+        case PROP_SWITCH_ENABLED:
+                gs_lock_plug_set_switch_enabled (self, g_value_get_boolean (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -363,6 +390,9 @@ gs_lock_plug_get_property (GObject    *object,
         case PROP_LOGOUT_ENABLED:
                 g_value_set_boolean (value, self->priv->logout_enabled);
                 break;
+        case PROP_SWITCH_ENABLED:
+                g_value_set_boolean (value, self->priv->switch_enabled);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -404,6 +434,13 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
                                                                NULL,
                                                                FALSE,
                                                                G_PARAM_READWRITE));
+        g_object_class_install_property (object_class,
+                                         PROP_SWITCH_ENABLED,
+                                         g_param_spec_boolean ("switch-enabled",
+                                                               NULL,
+                                                               NULL,
+                                                               FALSE,
+                                                               G_PARAM_READWRITE));
 }
 
 static gboolean
@@ -434,7 +471,7 @@ password_check_idle_cb (GSLockPlug *plug)
                                                                       (GSourceFunc)response_idle_cb,
                                                                       plug);
 
-                gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar), _("Password verification failed!"));
+                gtk_progress_bar_set_text (GTK_PROGRESS_BAR (plug->priv->progress_bar), _("Password check failed!"));
         }
 
         memset (local_password, '\b', strlen (local_password));
@@ -463,7 +500,7 @@ get_ok_button_for_page (gint page)
                 break;
         case (SWITCH_PAGE):
                 stock_id = GTK_STOCK_REFRESH;
-                label = N_("_Login Screen");
+                label = N_("_New Login");
                 break;
         default:
                 g_assert ("Invalid notebook page");
@@ -495,7 +532,7 @@ get_switch_button_for_page (gint page)
         switch (page) {
         case (AUTH_PAGE):
                 stock_id = GTK_STOCK_REFRESH;
-                label = N_("_Login Screen");
+                label = N_("_New Login");
                 break;
         case (SWITCH_PAGE):
                 stock_id = GTK_STOCK_DIALOG_AUTHENTICATION;
@@ -852,7 +889,7 @@ gs_lock_plug_init (GSLockPlug *plug)
         vbox = gtk_vbox_new (FALSE, 12);
         gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
 
-        widget = gtk_label_new (_("Enter password to unlock screen"));
+        widget = gtk_label_new (_("Enter a password to unlock the screen"));
         gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
         gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
 
@@ -945,8 +982,8 @@ gs_lock_plug_init (GSLockPlug *plug)
 
         gtk_label_set_attributes (GTK_LABEL (widget), pattrlist);
 
-        widget = gtk_label_new (_("This option will bring you to the Login Screen.\n"
-                                  "From the Login Screen you may log in to this system\n"
+        widget = gtk_label_new (_("This option will bring you to the login screen.\n"
+                                  "From the login screen you may log in to this system\n"
                                   "as another user or select 'Quit' to return to this screen."));
         gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
 
@@ -1012,6 +1049,8 @@ gs_lock_plug_init (GSLockPlug *plug)
 
         if (! plug->priv->logout_enabled)
                 gtk_widget_hide (plug->priv->logout_button);
+        if (! plug->priv->switch_enabled)
+                gtk_widget_hide (plug->priv->switch_button);
 
         plug->priv->timeout = 30000;
 
