@@ -58,6 +58,10 @@ GType       gs_window_get_type           (void);
 void        gs_window_set_screen         (GSWindow  *window,
                                           GdkScreen *screen);
 GdkScreen * gs_window_get_screen         (GSWindow  *window);
+void        gs_window_set_monitor        (GSWindow  *window,
+                                          int        monitor);
+int         gs_window_get_monitor        (GSWindow  *window);
+
 void        gs_window_set_lock_enabled   (GSWindow  *window,
                                           gboolean   lock_enabled);
 void        gs_window_set_logout_enabled (GSWindow  *window,
@@ -68,10 +72,12 @@ void        gs_window_set_logout_timeout (GSWindow  *window,
 void        gs_window_request_unlock     (GSWindow  *window);
 
 GSWindow  * gs_window_new                (GdkScreen *screen,
+                                          int        monitor,
                                           gboolean   lock_enabled);
 void        gs_window_show               (GSWindow  *window);
 void        gs_window_destroy            (GSWindow  *window);
 GdkWindow * gs_window_get_gdk_window     (GSWindow  *window);
+void        gs_window_clear              (GSWindow  *window);
 
 G_END_DECLS
 
